# SPDX-FileCopyrightText: (C) Gargantext Team <team@gargantext.org>
# SPDX-License-Identifier: CC0-1.0
{
  description = "A Nix Flake for Gargantext's Haskell server";
  # To use this Nix flake you may need to enable Nix flake support for your user with:
  #     echo >>~/.config/nix/nix.conf "experimental-features = nix-command flakes"
  # WARNING: be sure that `nix --version` is greater or equal to 2.18,
  # otherwise nix may not support some attributes used in flake.lock.
  
  # For an input, one can:
  # Update to the latest commit:
  #     nix flake lock --update-input nixpkgs
  # Or to a specific commit (eg. a green one on https://status.nixos.org):
  #     nix flake lock --override-input nixpkgs github:NixOS/nixpkgs/72da83d9515b43550436891f538ff41d68eecc7f
  # Or to a commit (in /etc/nix/registry.json) of a host NixOS:
  #     nix flake lock --override-input nixpkgs flake:nixpkgs
  inputs = {
    haskell-nix.url = "github:input-output-hk/haskell.nix";

    # For trying to hit cache.iog.io one would have
    # to follow haskell.nix's nixpkgs pinned version,
    # but it may be a few months old, so pin it here instead.
    #nixpkgs.follows = "haskell-nix/nixpkgs";
    nixpkgs.url = "github:NixOS/nixpkgs";
    haskell-nix.inputs.nixpkgs.follows = "nixpkgs";

    # Convenient Nix Flake utilities, like flake-utils.lib.eachSystem.
    flake-utils.url = "github:numtide/flake-utils";

    # FIXME: git.iscpif.fr's gitlab's API does not work well,
    # hence fallback to the git+https:// scheme.
    # gargantext-prelude.url = "gitlab:gargantext/haskell-gargantext-prelude?host=git.iscpif.fr";

    accelerate = { url = "git+https://github.com/AccelerateHS/accelerate.git?submodules=1"; flake = false; };
    accelerate-arithmetic = { url = "github:alpmestan/accelerate-arithmetic"; flake = false; };
    accelerate-llvm = { url = "github:AccelerateHS/accelerate-llvm"; flake = false; };
    # WARNING: using the fix-error branch
    accelerate-utility = { url = "git+https://gitlab.iscpif.fr/amestanogullari/accelerate-utility.git?ref=fix-error"; flake = false; };
    boolexpr = { url = "github:boolexpr/boolexpr"; flake = false; };

    coreNLP = { url = "http://nlp.stanford.edu/software/stanford-corenlp-4.5.4.zip"; flake = false; };
    crawlerArxiv = { url = "git+https://gitlab.iscpif.fr/gargantext/crawlers/arxiv-api.git"; flake = false; };
    crawlerHAL = { url = "git+https://gitlab.iscpif.fr/gargantext/crawlers/hal.git"; flake = false; };
    crawlerISTEX = { url = "git+https://gitlab.iscpif.fr/gargantext/crawlers/istex.git"; flake = false; };
    crawlerIsidore = { url = "git+https://gitlab.iscpif.fr/gargantext/crawlers/isidore.git"; flake = false; };
    crawlerPubMed = { url = "git+https://gitlab.iscpif.fr/gargantext/crawlers/pubmed.git"; flake = false; };
    data-time-segment = { url = "github:delanoe/data-time-segment"; flake = false; };
    eigen = { url = "github:chessai/eigen"; flake = false; };
    # WARNING: using the wavewave/ghcHEAD branch
    ekg-json = { url = "github:MercuryTechnologies/ekg-json/wavewave/ghcHEAD"; flake = false; };
    epo-api-client = { url = "git+https://gitlab.iscpif.fr/gargantext/crawlers/epo-proxy-api.git"; flake = false; };
    # WARNING: using the debugNaN branch
    gargantext-graph = { url = "git+https://gitlab.iscpif.fr/gargantext/gargantext-graph.git?ref=debugNaN"; flake = false; };
    # WARNING: unmerged commit
    # See https://gitlab.iscpif.fr/gargantext/haskell-gargantext-prelude/merge_requests/13
    gargantext-prelude = { url = "git+https://gitlab.iscpif.fr/gargantext/haskell-gargantext-prelude.git?ref=303-master-refactoring"; flake = false; };
    haskell-igraph = { url = "git+https://gitlab.iscpif.fr/gargantext/haskell-igraph.git"; flake = false; };
    hlcm = { url = "git+https://gitlab.iscpif.fr/gargantext/hlcm.git"; flake = false; };
    # WARNING: using the alp/imap-static branch
    hmatrix = { url = "github:alpmestan/hmatrix/alp/imap-static"; flake = false; };
    hsinfomap = { url = "git+https://gitlab.iscpif.fr/gargantext/haskell-infomap.git"; flake = false; };
    iso639 = { url = "git+https://gitlab.iscpif.fr/gargantext/iso639.git"; flake = false; };
    # WARNING: fork of https://github.com/llvm-hs/llvm-hs
    # using adinapoli/llvm-12-ghc-947-compat branch
    # Tracked in: https://gitlab.iscpif.fr/gargantext/haskell-gargantext/issues/318
    llvm-hs = { url = "github:adinapoli/llvm-hs/adinapoli/llvm-12-ghc-947-compat"; flake = false; };
    opaleye-textsearch = { url = "git+https://gitlab.iscpif.fr/gargantext/opaleye-textsearch.git?allRefs=1"; flake = false; };
    openalex = { url = "git+https://gitlab.iscpif.fr/gargantext/crawlers/openalex.git"; flake = false; };
    patches-class = { url = "git+https://gitlab.iscpif.fr/gargantext/patches-class.git"; flake = false; };
    patches-map = { url = "github:delanoe/patches-map"; flake = false; };
    rdf4h = { url = "github:robstewart57/rdf4h"; flake = false; };
    # WARNING: using the more-exports branch
    servant-job = { url = "github:alpmestan/servant-job/more-exports"; flake = false; };
    servant-xml-conduit = { url = "git+https://gitlab.iscpif.fr/gargantext/servant-xml-conduit"; flake = false; };
    # WARNING: using the alp/static branch
    sparse-linear = { url = "github:alpmestan/sparse-linear/alp/static"; flake = false; };
  };

  # For printing the available outputs:
  #     $ nix -L flake show --allow-import-from-derivation
  # Note that it is very slow the first time,
  # because this Flake's `checks` output, generate a build plan
  # for each Cabal test-suite of the project' closure.
  outputs = { self, ... }@inputs:
    let
      # FIXME: darwin support is untested.
      supportedSystems = with inputs.flake-utils.lib.system; [
        x86_64-linux
        #x86_64-darwin
        #aarch64-linux
        #aarch64-darwin
      ];
    in
      inputs.flake-utils.lib.eachSystem supportedSystems (system:
        let
          pkgs = import inputs.nixpkgs {
            inherit system;
            config = inputs.haskell-nix.config;
            overlays = [
              inputs.haskell-nix.overlay

              (finalPkgs: previousPkgs: {
                # FIXME: update to a version >= 10.0.1
                # to include 2a265497 from https://gitlab.com/graphviz/graphviz/-/merge_requests/3487
                graphviz = previousPkgs.graphviz.overrideAttrs (finalAttrs: previousAttrs: {
                  patches = (previousAttrs.patches or [ ]) ++ [
                    # Increase the YY_BUF_SIZE
                    # See https://gitlab.iscpif.fr/gargantext/haskell-gargantext/issues/290#note_9015
                    patches/graphviz/graphviz-yy-buf-size.diff
                  ];
                });
              })

              (finalPkgs: previousPkgs: {
                # Dependency of igraph and hmatrix
                openblas = previousPkgs.openblas.override {
                  # Let GHC handle the kernel threads.
                  # See also https://github.com/OpenMathLib/OpenBLAS/issues/2543
                  singleThreaded = true;
                };
                # WARNING: the nixpkgs pinned brings igraph-0.10.7 not igraph-0.10.4
                # as expected by haskell-igraph-0.10.4
                # Note that igraph uses the lapack and blas alternative selector packages
                # which default to using openblas.
                igraph = (previousPkgs.igraph.override {
                  # fop pulls openjdk, scipy, blas… just to build PDF docs,
                  fop = null;
                }).overrideAttrs (previousAttrs: {
                  # Note that blas and lapack in previousAttrs.buildInputs
                  # both default to wrapping openblas.
                  cmakeFlags = previousAttrs.cmakeFlags ++ [
                    # Disable Open Multi-Processing, to let GHC handle the kernel threads.
                    "-DIGRAPH_OPENMP_SUPPORT=OFF"
                  ];
                });
                # Dependency of igraph
                plfit = previousPkgs.plfit.overrideAttrs (previousAttrs: {
                  cmakeFlags = previousAttrs.cmakeFlags ++ [
                    # Disable Open Multi-Processing, to let GHC handle the kernel threads.
                    "-DPLFIT_USE_OPENMP=OFF"
                  ];
                });
              })

              (finalPkgs: previousPkgs: {
                pkgsCross = previousPkgs.pkgsCross // {
                  musl64 = previousPkgs.pkgsCross.musl64.extend (finalMusl64: previousMusl64: {
                    igraph = previousMusl64.igraph.overrideAttrs (previousAttrs: {
                      buildInputs = with finalMusl64; [
                        # Remove dependency on arpack (and eigen transitively)
                        blas # wraps openblas
                        glpk
                        gmp
                        lapack # wraps openblas
                        plfit
                      ];
                      cmakeFlags = previousAttrs.cmakeFlags ++ [
                        # Enable static build.
                        "-DBUILD_SHARED_LIBS=OFF"
                        # Using an external ARPACK does not work
                        # later on when linking haskell-igraph-test-test
                        # due to undefined references (eg. to dsortr_).
                        # Even when arpack is built with -DBUILD_SHARED_LIBS=OFF,
                        # is some other dependency not statically built? gfortran?
                        "-DIGRAPH_USE_INTERNAL_ARPACK=ON"
                        # Disable Link Time Optimization, because it fails with a lot of undefined references
                        # when linking statically here.
                        "-DIGRAPH_ENABLE_LTO=OFF"
                        # Disable Thread Local Storage, because it fails when linking gargantext-graph with:
                        # ghc-iserv: /nix/store/fpksda4yrj6khk20vkll403agicmnh5p-igraph-x86_64-unknown-linux-musl-0.10.7/lib/libigraph.a: unsupported internal ELF TLSGD relocation for symbol `igraph_i_finally_stack_size'
                        "-DIGRAPH_ENABLE_TLS=OFF"
                        #"-DIGRAPH_VERIFY_FINALLY_STACK=ON"
                        #"-DCMAKE_CXX_FLAGS=-fPIC"
                        #"-DCMAKE_C_FLAGS=-fPIC"
                      ];
                    });

                    openblas = previousMusl64.openblas.override {
                      # See https://github.com/input-output-hk/haskell.nix/issues/914#issuecomment-2021507590
                      enableStatic = true;
                      # Necessary to overcome lapack and blas packaging.
                      # And avoid ghc-iserv to fail finding the shared libs.
                      enableShared = true;
                    };

                    # Not enough anyway to avoid:
                    # unknown symbol `ZSTD_trace_compress_begin' in zstd Haskell package.
                    #zstd = previousMusl64.zstd.override {
                    #  static = true;
                    #};
                  });
                };
              })
            ];
          };

          # A standard library of Nix expressions.
          lib = inputs.nixpkgs.lib;

          # A library of Nix expressions internal to haskell.nix.
          inherit (pkgs.haskell-nix) haskellLib;

          # Pinning the index-state of Hackage,
          # instead of using the latest known by haskell.nix,
          # removes haskell.nix from interfering too much into the reproducibility.
          # It also enables to materialize the plan-nix.
          index-state = haskellLib.parseIndexState (lib.readFile ./cabal.project);

          compiler-nix-name = "ghc947";

          # Download GHC from Nixpkgs' binary cache instead of IOHK's
          # which would be done by using: p.haskell-nix.compiler
          # Fails with:
          # ghc-lib-ghc> Configuring library for ghc-9.4.7..
          # ghc-lib-ghc> genprimopcode: readCreateProcess: posix_spawnp: does not exist (No such file or directory)
          #compilerSelection = p: p.haskell.compiler;
          compilerSelection = p: p.haskell-nix.compiler;

          # findCabalFiles (in nix-tools/nix-tools/cabal2nix/Main.hs)
          # always prefers package.yaml over *.cabal,
          # but when the resulting *.cabal file is different
          # than a previously existing one,
          # the build fails with an error like this one:
          #     crawlerIsidore.cabal was modified manually, please use --force to overwrite.
          # Hence just remove this out-of-sync package.yaml.
          removePackageYaml = src: pkgs.symlinkJoin {
            name = "removePackageYaml-patched";
            paths = [ src ];
            postBuild = "rm $out/package.yaml";
            # Needed to retrive src.rev in local dependencyPackages
            passthru.src = src;
          };

          # Dependencies for which to produce a coverage report.
          coveredDependencies = pkgs: with pkgs; [
            accelerate
            accelerate-arithmetic
            accelerate-llvm
            accelerate-utility
            boolexpr
            crawlerArxiv
            crawlerHAL
            crawlerISTEX
            crawlerIsidore
            crawlerPubMed
            data-time-segment
            eigen
            ekg-json
            epo-api-client
            gargantext-graph
            gargantext-prelude
            # Alas, haskell-igraph uses .chs files that hpc does not consider.
            # Are there in some dist-newstyle somewhere that could be passed as --srcdir?
            #haskell-igraph
            hlcm
            hmatrix
            hsinfomap
            iso639
            llvm-hs
            opaleye
            openalex
            patches-class
            patches-map
            rdf4h
            servant-job
            servant-xml-conduit
            sparse-linear
          ];

          # Unfortunately project.hsPkgs.${pkgName}.components.library.srcSubDirPath
          # is not (cannot be?) set to the relevant srcSubDir
          # by haskell.nix/lib/import-and-filter-project.nix
          # for non-project packages.
          # Hence do it manually…
          srcSubDir = pkgName: {
            accelerate-llvm-native = "/${pkgName}";
            accelerate-llvm = "/${pkgName}";
            llvm-hs = "/${pkgName}";
            llvm-hs-pure = "/${pkgName}";
            hmatrix = "/packages/base";
            sparse-linear = "/${pkgName}";
          }.${pkgName} or "";

          # haskell.nix's main entry point
          project = pkgs.haskell-nix.cabalProject' {
            name = "gargantext";
            src = ./.;

            # By default plan evaluation is done on the build system.
            #evalSystem = "x86_64-linux";
            inherit compiler-nix-name;
            inherit compilerSelection;
            inherit index-state;

            # Materializing a project means caching
            # the nix files generated from the *.cabal/stack.yaml/package.yaml files.
            # To update:
            #     $ nix run .#update-nix-cache-plan
            # TODO: actually use it (and commit it in the git repo)
            # once every Gargantext developer understands it needs to be updated
            # when cabal.freeze changes.
            #materialized = nix/cache/plan;

            # Using inputMap for each source-repository-package of cabal.project
            # leverages Nix Flake's inputs to automatically get their sha256 hashes
            # and to check upstreams for updates (using `nix flake update`
            # or `nix flake lock --update-input <input>`).
            inputMap = {
              "https://github.com/AccelerateHS/accelerate-llvm.git" = inputs.accelerate-llvm;
              "https://github.com/AccelerateHS/accelerate.git" = inputs.accelerate;
              "https://github.com/MercuryTechnologies/ekg-json.git" = inputs.ekg-json;
              "https://github.com/boolexpr/boolexpr.git" = inputs.boolexpr;
              "https://github.com/adinapoli/llvm-hs.git" = inputs.llvm-hs;
              "https://github.com/alpmestan/accelerate-arithmetic.git" = inputs.accelerate-arithmetic;
              "https://github.com/alpmestan/hmatrix.git" = inputs.hmatrix;
              "https://github.com/alpmestan/servant-job.git" = inputs.servant-job;
              "https://github.com/alpmestan/sparse-linear.git" = inputs.sparse-linear;
              "https://github.com/chessai/eigen.git" = inputs.eigen;
              "https://github.com/delanoe/data-time-segment.git" = inputs.data-time-segment;
              "https://github.com/delanoe/patches-map" = inputs.patches-map;
              "https://gitlab.iscpif.fr/gargantext/opaleye-textsearch.git" = inputs.opaleye-textsearch;
              "https://github.com/robstewart57/rdf4h.git" = inputs.rdf4h;
              "https://gitlab.iscpif.fr/amestanogullari/accelerate-utility.git" = inputs.accelerate-utility;
              "https://gitlab.iscpif.fr/gargantext/crawlers/arxiv-api.git" = inputs.crawlerArxiv;
              "https://gitlab.iscpif.fr/gargantext/crawlers/epo-proxy-api.git" = inputs.epo-api-client;
              "https://gitlab.iscpif.fr/gargantext/crawlers/hal.git" = inputs.crawlerHAL;
              "https://gitlab.iscpif.fr/gargantext/crawlers/isidore.git" = inputs.crawlerIsidore;
              "https://gitlab.iscpif.fr/gargantext/crawlers/istex.git" = inputs.crawlerISTEX;
              "https://gitlab.iscpif.fr/gargantext/crawlers/openalex.git" = inputs.openalex;
              "https://gitlab.iscpif.fr/gargantext/crawlers/pubmed.git" = inputs.crawlerPubMed;
              "https://gitlab.iscpif.fr/gargantext/gargantext-graph.git" = inputs.gargantext-graph;
              "https://gitlab.iscpif.fr/gargantext/haskell-gargantext-prelude" = inputs.gargantext-prelude;
              "https://gitlab.iscpif.fr/gargantext/haskell-igraph.git" = inputs.haskell-igraph;
              "https://gitlab.iscpif.fr/gargantext/haskell-infomap.git" = inputs.hsinfomap;
              "https://gitlab.iscpif.fr/gargantext/hlcm.git" = inputs.hlcm;
              "https://gitlab.iscpif.fr/gargantext/iso639.git" = inputs.iso639;
              "https://gitlab.iscpif.fr/gargantext/patches-class.git" = inputs.patches-class;
              "https://gitlab.iscpif.fr/gargantext/servant-xml-conduit.git" = inputs.servant-xml-conduit;

              # Fixes needed for specific revisions
              # ===================================

              "https://github.com/alpmestan/accelerate-arithmetic.git/a110807651036ca2228a76507ee35bbf7aedf87a" = pkgs.applyPatches {
                name = "accelerate-arithmetic-patched";
                src = inputs.accelerate-arithmetic;
                patches = [ patches/accelerate-arithmetic/0001-remove-test-using-removed-realBandedGramian.patch ];
              };
              "https://gitlab.iscpif.fr/gargantext/crawlers/isidore.git/e9b594a4718acc06db037f6d3f429a90db76c267" = removePackageYaml inputs.crawlerIsidore;
              "https://gitlab.iscpif.fr/gargantext/haskell-gargantext-prelude/d4f9bee483d41bbdf8ab4e09efb5a5a8216edff4" = removePackageYaml inputs.gargantext-prelude;
              "https://gitlab.iscpif.fr/gargantext/crawlers/istex.git/c0a08d62c40a169b7934ceb7cb12c39952160e7a" = removePackageYaml inputs.crawlerISTEX;
            };

            # Default project configuration.
            modules = [
              ( { pkgs, ...}: {
                packages.haskell-igraph.components.library = {
                  # The generated plan includes pkgs.igraph
                  # but pkgs.igraph.dev is also needed for igraph.h
                  libs = [ pkgs.igraph.dev ];
                  # Extra include needed because haskell-igraph's cbits
                  # use: #include <igraph.h>
                  # not: #include <igraph/igraph.h>
                  configureFlags = [ "--extra-include-dirs=${pkgs.igraph.dev}/include/igraph" ];
                };

                # Link with OpenBLAS optimized libraries.
                # WARNING: OpenBLAS must only be used by trusted code
                # it is inherently unsuitable for security-conscious applications.
                # See nixpkgs/pkgs/development/libraries/science/math/openblas/default.nix
                packages.hmatrix.flags.openblas = true;
                # Not really necessary because nix builds in a sandbox by default anyway.
                packages.hmatrix.flags.disable-default-paths = true;

                # Start StanfordCoreNLPServer to pass the Hspec tests
                # This has the side effect of adding ~500MB to the build closure of gargantext…
                packages.gargantext.components.tests.garg-test-hspec.postPatch = ''
                  substituteInPlace test/drivers/hspec/Main.hs \
                    --replace 'proc (fromMaybe "./startServer.sh" <$> startServer) []' \
                              'proc "${pkgs.nixFlakes}/bin/nix" ["run", ".#coreNLP"]'
                '';

                packages.gargantext.components.tests.garg-test-hspec.testFlags = [
                  # FIXME:
                  # test/Test/Database/Operations.hs:64:7:
                  # 1) Database.Read/Writes, Corpus creation, Can add documents to a Corpus
                  #      uncaught exception: HttpException
                  #      HttpExceptionRequest Request {
                  #        host                 = "localhost"
                  #        port                 = 9000
                  #        secure               = False
                  #        requestHeaders       = [("Accept","application/json")]
                  #        path                 = "/"
                  #        queryString          = "?properties=%7B%22annotators%22:%22tokenize,ssplit,pos,ner%22,%22outputFormat%22:%22json%22%7D"
                  #        method               = "POST"
                  #        proxy                = Nothing
                  #        rawBody              = False
                  #        redirectCount        = 10
                  #        responseTimeout      = ResponseTimeoutDefault
                  #        requestVersion       = HTTP/1.1
                  #        proxySecureMode      = ProxySecureWithConnect
                  #      }
                  #       (ConnectionFailure Network.Socket.connect: <socket: 18>: does not exist (Connection refused))
                  "--match" "'!/Database/Read/Writes/Corpus creation/Can add documents to a Corpus/'"
                ];

                # Beware those failures are non-deterministic.
                # For reproducing it may help to use project's variants like coverage:
                #     $ nix -L build .#project.projectVariants.coverage.hsPkgs.gargantext.components.tests.garg-test-tasty
                #     $ for i in {1..100}; do result/bin/garg-test-tasty --hide-successes || break; done
                # Note that testFlags are only applied on checks.*, not on tests.*
                packages.gargantext.components.tests.garg-test-tasty.testFlags = [
                  "--pattern"
                  (lib.concatStringsSep " && " [
                    # FAIL: (non-deterministic)
                    # Graph Clustering
                    #     Cross
                    #       Partition test:                                                 FAIL (0.14s)
                    #         uncaught exception: ErrorCall
                    #         *** Internal error in package accelerate ***
                    #         *** Please submit a bug report at https://github.com/AccelerateHS/accelerate/issues
                    #         Encountered a duplicate 'Tag'
                    #           81: Tag 1, 77: Tag 1, 79: Tag 1, 84: Tag 1
                    #         CallStack (from HasCallStack):
                    #           internalError: Data.Array.Accelerate.Trafo.Sharing:2274:19
                    #           buildInitialEnvExp: Data.Array.Accelerate.Trafo.Sharing:2702:17
                    #           determineScopesExp: Data.Array.Accelerate.Trafo.Sharing:2608:17
                    #           scopesExp: Data.Array.Accelerate.Trafo.Sharing:2636:26
                    #           scopesFun1: Data.Array.Accelerate.Trafo.Sharing:2429:60
                    #           scopesAcc: Data.Array.Accelerate.Trafo.Sharing:2490:33
                    #           travF2MEA: Data.Array.Accelerate.Trafo.Sharing:2406:38
                    #           scopesAcc: Data.Array.Accelerate.Trafo.Sharing:2464:33
                    #           travEA: Data.Array.Accelerate.Trafo.Sharing:2397:38
                    #           scopesAcc: Data.Array.Accelerate.Trafo.Sharing:2509:34
                    #           travF2A2: Data.Array.Accelerate.Trafo.Sharing:2405:38
                    #           scopesAcc: Data.Array.Accelerate.Trafo.Sharing:2490:33
                    #           travF2MEA: Data.Array.Accelerate.Trafo.Sharing:2406:38
                    #           scopesAcc: Data.Array.Accelerate.Trafo.Sharing:2490:33
                    #           travF2MEA: Data.Array.Accelerate.Trafo.Sharing:2406:38
                    #           scopesAcc: Data.Array.Accelerate.Trafo.Sharing:2331:46
                    #           determineScopesSharingAcc: Data.Array.Accelerate.Trafo.Sharing:2718:17
                    #           scopesAcc: Data.Array.Accelerate.Trafo.Sharing:2818:33
                    #           travAE: Data.Array.Accelerate.Trafo.Sharing:2767:36
                    #           scopesExp: Data.Array.Accelerate.Trafo.Sharing:2790:32
                    #           travE2: Data.Array.Accelerate.Trafo.Sharing:2749:36
                    #           scopesExp: Data.Array.Accelerate.Trafo.Sharing:2780:30
                    #           travE1: Data.Array.Accelerate.Trafo.Sharing:2766:36
                    #           scopesExp: Data.Array.Accelerate.Trafo.Sharing:2789:32
                    #           travE2: Data.Array.Accelerate.Trafo.Sharing:2749:36
                    #           scopesExp: Data.Array.Accelerate.Trafo.Sharing:2780:30
                    #           travE1: Data.Array.Accelerate.Trafo.Sharing:2751:36
                    #           scopesExp: Data.Array.Accelerate.Trafo.Sharing:2800:32
                    #           travE3: Data.Array.Accelerate.Trafo.Sharing:2760:36
                    #           scopesExp: Data.Array.Accelerate.Trafo.Sharing:2712:56
                    #           determineScopesSharingExp: Data.Array.Accelerate.Trafo.Sharing:2696:60
                    #           determineScopesExp: Data.Array.Accelerate.Trafo.Sharing:2608:17
                    #           scopesExp: Data.Array.Accelerate.Trafo.Sharing:2636:26
                    #           scopesFun1: Data.Array.Accelerate.Trafo.Sharing:2400:60
                    #           scopesAcc: Data.Array.Accelerate.Trafo.Sharing:2331:46
                    #           determineScopesSharingAcc: Data.Array.Accelerate.Trafo.Sharing:2317:37
                    #           determineScopesAcc: Data.Array.Accelerate.Trafo.Sharing:3101:5
                    #           recoverSharingAcc: Data.Array.Accelerate.Trafo.Sharing:240:34
                    #           convertOpenAcc: Data.Array.Accelerate.Trafo.Sharing:162:35
                    #           convertAccWith: Data.Array.Accelerate.Trafo:71:37
                    "'!/Partition test/'"

                    # FIXME: FAIL (non-deterministic)
                    #     Starting 1 job runners.
                    #     Starting 1 job runners.
                    #     expected: [Nothing,Just 10,Just 5]
                    #      but got: [Nothing,Nothing,Nothing]
                    "!/can fetch the latest job status/"

                    # FIXME: FAIL (non-deterministic)
                    # expected: [Just 100]
                    #  but got: [Nothing]
                    # expected: [Just 50]
                    #  but got: [Nothing]
                    "'!/can spin two separate jobs and track their status separately/'"
                  ])
                ];
              })
            ];

            # Variants to the default project configuration above.
            # They're accessed in the flake's outputs with their name prefixed.
            #     $ nix -L build .#static-x86_64-unknown-linux-musl:gargantext:exe:gargantext-phylo-profile
            # Or via project.projectVariants:
            #     $ nix -L build .#project.projectVariants.static.projectCross.musl64.hsPkgs.postgresql-libpq.components.library
            flake.variants = {
              profiling = {
                modules = [
                  {
                    # Applies to all packages of the Haskell closure. For instance:
                    #     $ nix eval .#project.hsPkgs.containers.components.library.config.enableProfiling
                    #     false
                    #     $ nix eval .#project.projectVariants.profiling.hsPkgs.containers.components.library.config.enableProfiling
                    #     true
                    enableProfiling = true;
                    enableLibraryProfiling = true;
                  }
                ];
              };

              # For building static executables:
              #     $ nix -L build .#static-x86_64-unknown-linux-musl:gargantext:exe:gargantext-phylo-profile
              #
              # Note the use of the Musl cross compiling platform
              # to replace glibc which does not fully support static linking.
              #
              # For debugging static executables (Musl is not bug-to-bug compatible with glibc):
              #     $ nix -L develop .#legacyPackages.x86_64-linux.project.projectVariants.static.projectCross.musl64.hsPkgs.gargantext-graph.components.library
              #     devShell> $ rm -rf source outputs
              #     devShell> $ unpackPhase
              #     devShell> $ cd source
              #     devShell> $ patchPhase
              #     devShell> $ eval "$configurePhase"
              #     devShell> $ eval "$buildPhase"
              #     devShell> $ (unset LD_LIBRARY_PATH; $EDITOR **/*.hs) # hack hack hack
              #     devShell> $ eval "$buildPhase"
              #     devShell> $ eval "$installPhase"
              #
              # FIXME: this static compiling currently fails.
              # A problem to look for is pkgsCross.musl64
              # still having some libraries not built statically.
              static = {
                modules = [
                  ({ config, pkgs, ... }: {
                    enableStatic = true;
                    # Fails with:
                    # cmdargs-lib-cmdargs-x86_64-unknown-linux-musl>     Bad interface file: /nix/store/n0iwpy7240kw1mkmryzcgj9s9vvbfslb-transformers-lib-transformers-x86_64-unknown-linux-musl-0.5.6.2/lib/x86_64-linux-ghc-9.4.7/transformers-0.5.6.2-7uipldXxD6oGHq4y3fP618/Control/Monad/Trans/State.dyn_hi
                    #
                    # Besides it does not apply DYNAMIC_GHC_PROGRAMS=NO to GHC.
                    #enableShared = false;

                    # Musl does not provide the thread-safe random_r()/initstate_r()
                    # so make randomVector fallback to a **non** thread-safe alternative, nrand48()
                    # See https://github.com/haskell-numerics/hmatrix/issues/279
                    #
                    # As a consequence, random numbers may not be random between threads
                    # as they now share a common state.
                    # See http://www.evanjones.ca/random-thread-safe.html
                    # > the best solution for truly portable applications
                    # > is to include your own random number generator implementation,
                    # > and not rely on the system's C library.
                    # See https://github.com/haskell-numerics/hmatrix/issues/307
                    packages.hmatrix.flags.no-random_r = true;

                    # Use pkg-config instead of pg_config to avoid:
                    #     $ nix -L build .#legacyPackages.x86_64-linux.project.projectVariants.static.projectCross.musl64.hsPkgs.postgresql-libpq.components.library
                    #     > Error: Setup: The program 'pg_config' is required but it could not be found
                    packages.postgresql-libpq.flags.use-pkg-config = true;

                    # Fix c2hs failing to run ghc
                    # because when cross-compiling it's prefixed, eg. x86_64-unknown-linux-musl-ghc
                    # TODO: remove when https://github.com/haskell/c2hs/pull/293 has been merged
                    packages.c2hs.components.exes.c2hs.depends = [
                      config.hsPkgs.ghc-paths
                    ];
                    packages.c2hs.patches = [
                      patches/c2hs/0001-fix-GHC-path-to-support-cross-compiling.patch
                    ];
                    # Alas, line-ending mess on c2hs.cabal prevents to use a patch.
                    # And patchFlags=["--binary"] is not passed to mkDerivation by haskell.nix
                    # Hence use substituteInPlace to add ghc-paths to the build-depends.
                    packages.c2hs.postPatch = ''
                      substituteInPlace c2hs.cabal \
                        --replace "base >= 2 && < 5," "base >= 2 && < 5, ghc-paths,"
                    '';

                    # Musl does not support dlopen().
                    # But GHC's RTS supports internal_dlopen() when -fexternal-dynamic-refs is used,
                    # as enabled in haskell.nix/compiler/ghc/default.nix
                    # however it may fail to do so (with a SIGSEGV)
                    # when splicing some TemplateHaskell code.
                    # See https://gitlab.haskell.org/ghc/ghc/-/issues/20628#note_393505
                    #
                    # Another alternative is to use -fexternal-interpreter which spawns a ghc-iserv,
                    # sadly it slows down the compiling by a huge factor…
                    packages.accelerate-llvm.ghcOptions = [ "-fexternal-interpreter" ];
                    packages.accelerate-llvm-native.ghcOptions = [ "-fexternal-interpreter" ];

                    # gargantext-graph uses TemplateHaskell.
                    packages.gargantext-graph.ghcOptions = [ "-fexternal-interpreter" ];

                    # Due to unknown symbol __cpu_model in bitvec-1.1.5.0
                    # when using -fexternal-interpreter,
                    # the SIMD optimization has to be disabled.
                    # See https://github.com/input-output-hk/haskell.nix/issues/2047
                    packages.bitvec.flags.simd = false;

                    # FIXME:
                    # ghc-iserv: /nix/store/nrycyind0m4lfn34b13h68bq2scj9vrk-zstd-lib-zstd-x86_64-unknown-linux-musl-0.1.3.0/lib/x86_64-linux-ghc-9.4.7/zstd-0.1.3.0-19JqsNNxPIf7nO0TGXWXpF/HSzstd-0.1.3.0-19JqsNNxPIf7nO0TGXWXpF.o: unknown symbol `ZSTD_trace_compress_begin'
                    # ghc-iserv: Could not load Object Code /nix/store/nrycyind0m4lfn34b13h68bq2scj9vrk-zstd-lib-zstd-x86_64-unknown-linux-musl-0.1.3.0/lib/x86_64-linux-ghc-9.4.7/zstd-0.1.3.0-19JqsNNxPIf7nO0TGXWXpF/HSzstd-0.1.3.0-19JqsNNxPIf7nO0TGXWXpF.o.
                    # <no location info>: error: unable to load unit `zstd-0.1.3.0'
                    # <no location info>: error: unable to load unit `ghc-prim-0.9.1'
                    packages.gargantext.ghcOptions = [
                      "-fexternal-interpreter"
                      #"-optl=-L${lib.getLib pkgs.zstd}/lib"
                      #"-optl=-lzstd"
                    ];
                    packages.gargantext.components.library.libs = [
                      #(lib.getLib pkgs.zstd)
                    ];
                  })
                ];
                flake.crossPlatforms = platforms:
                  pkgs.lib.optionals pkgs.stdenv.hostPlatform.isx86_64 (
                    # For building static executables.
                    pkgs.lib.optionals pkgs.stdenv.hostPlatform.isLinux [
                      platforms.musl64
                    ]
                    # For cross compiling to Windows.
                    # ++ [ platforms.mingwW64 ]
                  );
              };

              coverage = {
                modules = [
                  ({ pkgs, ... }: {
                    # For generating the project coverage report:
                    #     $ nix -L build .#coverageReport
                    #     $ firefox result/share/hpc/vanilla/html/index.html
                    packages.gargantext.components.library.doCoverage = true;

                    # haskell-igraph uses .chs files that hpc does not find…
                    packages.haskell-igraph.components.library.doCoverage = lib.mkForce false;
                  })
                  {
                    packages = lib.genAttrs
                      (map (pkg: pkg.identifier.name) (coveredDependencies project.hsPkgs))
                      (_: { doCoverage = true; });
                  }
                ];
              };
            };

            # Shell configuration shared by the default shell
            # and all shells from the flake.variants.
            shell = {
              # Force (by setting CABAL_CONFIG=) cabal-install to use the packages
              # selected by project.plan-nix.
              exactDeps = true;

              # Provision (in `ghc-pkg list`) the **dependencies** of the packages selected here,
              # which are not also selected here.
              # By default haskell.nix selects all _local_ packages here
              # (packages from both the `packages` and the source-repository-package stanzas)
              # which therefore excludes source-repository-packages from being provisioned.
              # Hence select only the _project_ packages (packages from the `packages` stanza).
              packages = ps: lib.attrValues (haskellLib.selectProjectPackages ps);

              # Add in this list any development tool needed
              # that is not expected to come from the developers' own system.
              nativeBuildInputs = [
                # Wraps cabal-install to remove source-repository-package stanzas,
                # otherwise it would download an use them,
                # likely because it does not know that those packages provisioned
                # in `ghc-pkg list` must be used in priority.
                (let cabal-install = project.tool "cabal" {
                  version = "latest";
                  #inherit index-state;
                  modules = [
                    {
                      # Preserve Bash completion for cabal.
                      packages.cabal-install.components.exes.cabal.postInstall = ''
                        mkdir -p $out/share/bash-completion
                        mv bash-completion $out/share/bash-completion/completions
                      '';
                    }
                  ];
                };
                in
                # symlinkJoin cabal-install to preserve share/bash-completion/
                pkgs.symlinkJoin {
                  name = "cabal-install";
                  paths = [ cabal-install ];
                  nativeBuildInputs = [ pkgs.makeWrapper ];
                  # Wrap cabal-install to make it use the local cabal.project.flake.
                  postBuild = ''
                    rm "$out"/bin/cabal
                    makeWrapper \
                      ${cabal-install}/bin/cabal \
                      "$out"/bin/cabal \
                      --add-flags --project-file=cabal.project.flake
                  '';
                })

                (project.tool "ghcid" { version = "latest"; })

                # Provide HLS because it is tighly coupled to the GHC version in use.
                # TODO: uncomment when in public binary caches.
                #(project.tool "haskell-language-server" { version = "latest"; })

                (project.tool "hlint" { version = "latest"; })
              ];

              # cabal.project.flake must be in the same directory than the original cabal.project
              # for the relative paths in the packages stanza to point to the correct subdirs.
              shellHook = ''
                export GARGANTEXT_CORENLP_SERVER="nix run .#coreNLP"
                ln -sf ${pkgs.runCommandLocal "cabal.project" {
                      nativeBuildInputs = with pkgs; [ gawk ];
                    }
                    # Strips source-repository-package stanzas from cabal.project.
                    ''
                      awk '/^[^ ]/{f=/^source-repository-package/} !f' >$out ${./cabal.project}
                    ''
                  } cabal.project.flake
              '';

              # When true, builds a Hoogle documentation index of all dependencies,
              # and provides a "hoogle" command to search the index.
              withHoogle = true;
            };
          };

          # Packages for which a build plan cannot be computed by cabal-install.
          brokenPlans = [
            # error: The option `packages.GLFW-b.package.identifier.name' is used but not defined.
            "GLFW-b"
            # error: The option `packages.Sit.package.identifier.name' is used but not defined.
            "Sit"
            # error: The option `packages.X11.package.identifier.name' is used but not defined.
            "X11"
            # error: The option `packages.Win32.package.identifier.name' is used but not defined.
            "Win32"
            # fail (backjumping, conflict set: binary, hashable, text, unordered-containers)
            "binary"
            # error: The option `packages.bindings-GLFW.package.identifier.name' is used but not defined.
            "bindings-GLFW"
            # [__3] rejecting: random-1.2.1 (cyclic dependencies; conflict set: bytestring, random, tasty-quickcheck)
            "bytestring"
            # [__3] rejecting: ansi-terminal-0.11.5 (cyclic dependencies; conflict set: ansi-terminal, colour, test-framework)
            "colour"
            # error: The option `packages.discount.package.identifier.name' is used but not defined.
            "discount"
            # [__3] rejecting: these-1.2 (cyclic dependencies; conflict set: foldable1-classes-compat, quickcheck-instances, these)
            "foldable1-classes-compat"
            # Could not resolve dependencies: [__0] next goal: ghc (user goal)
            "ghc"
            "ghc-boot"
            "ghc-heap"
            "ghc-lib-parser"
            "ghc-syntax-highlighter"
            "ghci"
            # error: The option `packages.gi-gtk.package.identifier.name'
            "gi-gtk"
            # conflict set: http-api-data, http-api-data:hspec-discover:exe.hspec-discover, http-api-data:test
            "http-api-data"
            "libiserv"
            # error: The option `packages.mintty.package.identifier.name'
            "mintty"
            # error: The option `packages.odbc.package.identifier.name' is used but not defined.
            "odbc"
            # error: The option `packages.pcap.package.identifier.name' is used but not defined.
            "pcap"
            # fail (backjumping, conflict set: ghc, tasty-inspection-testing, time)
            "random"
            # fail (backjumping, conflict set: base, cassava, scientific)
            "scientific"
            # [__2] fail (backjumping, conflict set: servant-client, servant-client:hspec-discover:exe.hspec-discover, servant-client:test)
            "servant-client"
            # fail (backjumping, conflict set: base, ekg, servant-ekg)
            "servant-ekg"
            # [__2] fail (backjumping, conflict set: servant-foreign, servant-foreign:hspec-discover:exe.hspec-discover, servant-foreign:test)
            "servant-foreign"
            # [__2] fail (backjumping, conflict set: servant-swagger, servant-swagger:hspec-discover:exe.hspec-discover, servant-swagger:test)
            "servant-swagger"
            # fail (backjumping, conflict set: QuickCheck, aeson, criterion, criterion-measurement, splitmix)
            "splitmix"
            # [__2] fail (backjumping, conflict set: swagger2, swagger2:hspec-discover:exe.hspec-discover, swagger2:test)
            "swagger2"
            # fail (backjumping, conflict set: criterion, time, time:bench)
            "time"
            # fail (backjumping, conflict set: optparse-applicative, process, tasty, tasty-hunit, template-haskell, unix, optparse-applicative:process)
            "unix"
          ];

          # Package for each dependency of the given config of the project.
          dependencyPackages = config: lib.mapAttrs (packageName: package:
            if package != null
            && (package.isHaskell or false)
            # Already provided by projectFlake.checks
            && !(package.isProject or false)
            # Some packages cannot be used:
            # error: The option `packages.BNFC.package.identifier.name' is used but not defined.
            && (package.components ? "library")
            && !(lib.elem packageName brokenPlans)
            then
              # This only includes the source-repository-packages
              # since the project packages have been filtered-out above.
              if config.hsPkgs.${packageName}.isLocal
              then ((pkgs.haskell-nix.cabalProject' (config.projectModule // {
                  name = packageName;
                  # Some source-repository-packages have a subdir stanza.
                  src = config.hsPkgs.${packageName}.src + srcSubDir packageName;
                  configureArgs = "--disable-benchmarks --minimize-conflict-set";
                  # Reuse the exact same dependency versions as the project,
                  # for the dependencies they share.
                  cabalProjectFreeze = lib.readFile ./cabal.project.freeze;
                  cabalProject = let
                    source-repository-packages = haskellLib.parseSourceRepositoryPackages "cabal.project" {} {} (lib.readFile ./cabal.project);
                  in ''
                    packages: ./
                    allow-newer: *
                    allow-older: *
                  ''
                  # Add cabal.project's source-repository-packages
                  # except the one for this package.
                  + lib.concatMapStringsSep "\n" (pkg:
                      let pkgSourceRepo = (lib.head (lib.filter
                            # Using pkg.src.src.rev is to handle the case when
                            # applyPatches or symlinkJoin has been used on an input.
                            (block: block.sourceRepo.rev == pkg.src.src.rev or pkg.src.rev)
                            source-repository-packages.sourceRepos)).sourceRepo; in
                    ''
                      source-repository-package
                        type: git
                        location: ${pkgSourceRepo.url}
                        subdir: ${lib.concatStringsSep " " pkgSourceRepo.subdirs}
                        tag: ${pkgSourceRepo.rev}
                    ''
                  ) (lib.attrValues (lib.filterAttrs
                      (pkgName: pkg:
                        !(pkg.isProject or false) &&
                        (pkg.isLocal or false) &&
                        # This whole parsing of source-repository-packages
                        # is to remove pkgName from the source-repository-packages,
                        # otherwise it confuses cabal-install.
                        pkgName != packageName)
                    config.hsPkgs))
                  ;
                  /* Unfortunately, this does not work as well as the above, it fails for:
                         nix -L build .#dependencyPackages.accelerate-arithmetic.components.tests.test
                  pkg-def-extras =
                    lib.attrValues (lib.filterAttrs (pkgName: pkg:
                      (hsPkgs.${pkgName}.isLocal or false) && pkgName != packageName)
                      project.pkg-set.config.packages);
                  */
                })).appendModule {
                  modules = [
                    {
                      # haskell.nix's applyComponents filters-out non-planned components,
                      # causing tests to be filtered-out.
                      planned = true;
                    }
                  ];
                }).getPackage packageName
              # Other dependencies are fetched from Hackage.
              else pkgs.haskell-nix.hackage-package {
                name = package.identifier.name;
                version = package.identifier.version;
                revision = package.identifier.revision or "default";
                inherit compiler-nix-name;
                inherit compilerSelection;
                inherit index-state;
                # By default haskell.nix uses --enable-tests and --enable-benchmarks
                configureArgs = "--disable-benchmarks --minimize-conflict-set";
                cabalProjectFreeze = lib.readFile ./cabal.project.freeze;
                cabalProject = ''
                  packages: ./
                  allow-newer: *
                  allow-older: *
                '';
              }
            else null
          ) config.hsPkgs;

          coverageReport =
            let proj = project.projectVariants.coverage; in
            # project.components.library and coveredDependencies coverage by gargantext's test-suites.
            (haskellLib.projectCoverageReport proj
              (map
                (pkg:
                  haskellLib.coverageReport ({
                    name = pkg.identifier.name + "-" + pkg.identifier.version;
                    ghc = proj.pkg-set.config.ghc.package;
                    # Focus only on the coverage of the library of pkg.
                    mixLibraries = [ (pkg.components.library // {
                      srcSubDirPath = pkg.components.library.srcSubDirPath + srcSubDir pkg.identifier.name;
                    }) ];
                    # Using the test-suites of gargantext instead of the test-suites of pkg.
                    checks = lib.concatMap
                      # Prefix check names with pkg.identifier.name to avoid name conflicts
                      # since the same checks will be run for each package.
                      (check: lib.optional (lib.isDerivation check)
                        (check // { name = "${pkg.identifier.name}/${check.name}"; }))
                      (lib.attrValues proj.hsPkgs.gargantext.checks);
                  }))
                (coveredDependencies proj.hsPkgs))
            ).overrideAttrs (previousAttrs: {
              buildCommand = previousAttrs.buildCommand + ''
                substituteInPlace $out/share/hpc/vanilla/html/index.html \
                  --replace "The following reports show how the tests of each package cover modules in the project:" \
                            "The following reports show how the tests of gargantext cover modules in some dependencies:"
                ( cd $out/share/hpc/vanilla/html ; zip -rf $out/share/hpc/vanilla/html.zip . )
              '';
            });

          # coveredDependencies coverage by their own test-suites.
          # TODO: test
          dependenciesOwnCoverageReport =
            let depPkgs = dependencyPackages project; in
            haskellLib.projectCoverageReport
              (project // {
                hsPkgs = project.hsPkgs //
                  lib.genAttrs
                    (map (pkg: pkg.identifier.name) (coveredDependencies depPkgs))
                    (name: depPkgs.${name} // { isProject = true; });
              })
              (map (pkg:
                  haskellLib.coverageReport ({
                    name = pkg.identifier.name + "-" + pkg.identifier.version;
                    ghc = project.pkg-set.config.ghc.package;
                    # Focus only on the coverage of the library of pkg.
                    mixLibraries = [ pkg.components.library ];
                    # Using the test-suites of pkg.
                    checks = lib.filter lib.isDerivation (lib.attrValues pkg.checks);
                  }))
                (coveredDependencies depPkgs)
              );

          projectFlake = project.flake { };
        in {
          legacyPackages = pkgs // {
            dependencyPackages = dependencyPackages project;
            # For exploring the project:
            #     $ nix --extra-experimental-features 'flakes repl-flake' repl .
            #     nix-repl> :lf .
            #     nix-repl> legacyPackages.x86_64-linux.project.<TAB>
            inherit project;
          };

          # For building a component of this project:
          #     $ nix -L build .#gargantext:exe:gargantext-phylo-profile
          packages = /*projectFlake.packages //*/ {
            # Not including all projectFlake.packages because
            # anything querying the package list,
            # like using .#project instead of .#legacyPackages.x86_64-linux.project
            # or just `nix develop`, would list its attribute names and force to build
            # too much of the supported projectCross plaforms, like musl64.
            # Besides, not all combinations of projectCross and projectVariants are actually meaningful.
            inherit (projectFlake.packages)
              "gargantext:bench:garg-bench"
              "gargantext:exe:gargantext-admin"
              "gargantext:exe:gargantext-cli"
              "gargantext:exe:gargantext-db-obfuscation"
              "gargantext:exe:gargantext-import"
              "gargantext:exe:gargantext-init"
              "gargantext:exe:gargantext-invitations"
              "gargantext:exe:gargantext-phylo"
              "gargantext:exe:gargantext-phylo-profile"
              "gargantext:exe:gargantext-server"
              "gargantext:exe:gargantext-upgrade"
              "gargantext:lib:gargantext"
              "gargantext:test:garg-test-hspec"
              "gargantext:test:garg-test-tasty"
              "static-x86_64-unknown-linux-musl:gargantext:exe:gargantext-phylo-profile";
            inherit coverageReport;
          };

          # For entering the default development shell:
          #     $ nix -L develop
          #     $ cabal build --disable-optimization
          #
          # For entering the development shell variant `profiling`:
          #     $ nix -L develop .#profiling
          #     $ cabal run --enable-profiling gargantext-phylo-profile
          devShells = projectFlake.devShells;

          apps = projectFlake.apps // {

            # For updating nix/cache/plan
            #     $ nix run .#update-nix-cache-plan
            #     $ git commit -m "nix: update nix/cache/plan"
            update-nix-cache-plan = {
              type = "app";
              program = (pkgs.writeShellScript "update-nix-cache-plan" ''
                set -x
                ${project.plan-nix.passthru.generateMaterialized} nix/cache/plan
                git add --all nix/cache/plan
              '').outPath;
            };

            # Register the project's toolchain for compiler-nix-name
            # (but not the Haskell package dependencies of the project)
            # to prevent nix-collect-garbage from removing them from the Nix store.
            register-roots = {
              type = "app";
              program = (pkgs.writeShellScript "register-roots" ''
                nix-store --add-root nix/cache/roots --indirect --realise ${project.roots}
              '').outPath;
            };

            coreNLP = {
              type = "app";
              program = (pkgs.writeShellScript "coreNLP" ''
                set -x
                exec ${pkgs.openjdk}/bin/java -mx4g -cp '${inputs.coreNLP}/*' edu.stanford.nlp.pipeline.StanfordCoreNLPServer -port 9000 -timeout 15000
              '').outPath;
            };
          };

          # For running all checks (very slow):
          #     $ nix -L flake check
          # FIXME: actually run them all and disable or fix the broken ones.
          #
          # For building a specific check of the project:
          #     $ nix -L build .#project.hsPkgs.gargantext.components.tests.garg-test-tasty
          #     $ result/bin/garg-test-tasty
          #
          # For building a specific check of the dependencies:
          #     $ nix -L build .#dependencyPackages.duckling.components.tests.duckling-test
          #     $ result/bin/duckling-test
          #
          # Alternatively, but slower:
          #     $ nix -L build .#checks.x86_64-linux.duckling:test:duckling-test
          #     $ bat result/test-stdout
          #
          # See names from:
          #     $ nix -L flake show --allow-import-from-derivation
          checks = projectFlake.checks // (
            # Generate a check for each test-suite
            # of each package in the closure of this project.
            haskellLib.mkFlakeChecks (haskellLib.collectChecks' (dependencyPackages project))
          );

          #hydraJobs = projectFlake.hydraJobs;
        }
      );

  # Ask users to set Nix config entries in its local nix.conf.
  nixConfig = {
    # This sets the flake to use the IOG nix cache.
    # nix asks for permission before using it.
    # It's not currently needed because compilerSelection uses Nixpkgs' GHC.
    extra-substituters = [
      "https://cache.iog.io"
    ];
    extra-trusted-public-keys = [
      "hydra.iohk.io:f/Ea+s+dFdN+3Y/G+FDgSq+a5NEWhJGzdjvKNGv0/EQ="
    ];

    # haskell.nix translates to Nix expressions:
    # - the build plan usually generated in `dist-newstyle/cache/plan.json` by `cabal configure`
    # - and the `.cabal`/`stack.yaml`/`package.yaml` files of projects.
    #
    # haskell.nix can either generate those Nix expressions on-demand
    # by calling its nix-tools' make-install-plan and cabal-to-nix,
    # hence importing them from a derivation (IFD).
    # Or import pre-generated files whenever project's materialized= attribute is set,
    # andthen no longer needs to allow IFD.
    #
    # Note that the projects for the checks of the closure in dependencyPackages
    # are never materialized, hence checking or showing the checks
    # always requires to allow IFD.
    allow-import-from-derivation = "true";
  };
}
